@@ -1,13 +1,15 @@
 use std::ffi::c_void;
-use std::ffi::CStr;
 use std::ffi::CString;
 use std::os::raw::c_char;
 
+use crate::tcl::*;
 use crate::Object;
+use crate::ObjectType;
+use crate::RawObject;
 
-/// A wrapper around a [TCL](https://www.tcl.tk) interpreter object.
+/// A wrapper around a [Tcl](https://www.tcl.tk) interpreter object.
 ///
-/// This is a wrapper around the TCL interpreter object that leverages the
+/// This is a wrapper around the Tcl interpreter object that leverages the
 /// Stubs interface.  It makes assumptions about the interface (such as
 /// stability of function pointers) that are made by the underlying stubs
 /// implementation.  However, as a deliberate nod to the importance of the
@@ -17,6 +19,7 @@ use crate::Object;
 /// (extra indirection), it fits more with Rust paradigms and should reduce
 /// the risk of trying to use the API without an associated interpeter.
 #[repr(C)]
+#[derive(Debug)]
 pub struct Interpreter {
     _legacy_result: *const c_void,
     _legace_free_proc: *const c_void,
@@ -26,11 +29,11 @@ pub struct Interpreter {
 
 type CmdProc = fn(interp: &Interpreter, args: Vec<&str>) -> Result<TclStatus, String>;
 
-const TCL_STUB_MAGIC: u32 = 0xFCA3BACF; // TCL 8.x extension
+const TCL_STUB_MAGIC: u32 = 0xFCA3BACF; // Tcl 8.x extension
 
-/// A wrapper for TCL return status codes.
+/// A wrapper for Tcl return status codes.
 ///
-/// This is a simple wrapper around the expected return codes for TCL
+/// This is a simple wrapper around the expected return codes for Tcl
 /// commands.  `Ok` and `Error` are the most common ones, but the others have
 /// specific meanings under certain conditions (e.g., binding handlers in
 /// Tk).  See the appropriate documentation for specific behavior.
@@ -44,6 +47,17 @@ pub enum TclStatus {
     Continue = 4,
 }
 
+/// A wrapper for values passed to Tcl's [unload](https://www.tcl.tk/man/tcl/TclCmd/unload.html) function.
+#[repr(isize)]
+pub enum TclUnloadFlag {
+    /// Inidicates the interpreter is exiting but that the module's code is
+    /// not being unmapped.
+    DetachFromInterpreter = 1 << 0,
+    /// Inidicates the last interpreter is detaching and the module is about
+    /// to be unmapped from the process.
+    DetachFromProcesss = 1 << 1,
+}
+
 const _TCL_STATIC: *const c_void = 0 as *const c_void;
 const _TCL_VOLATILE: *const c_void = 1 as *const c_void;
 const TCL_DYNAMIC: *const c_void = 3 as *const c_void;
@@ -57,7 +71,7 @@ struct Stubs {
     _untranslated_function1: *const c_void,  // 1
     _untranslated_function2: *const c_void,  // 2
     _untranslated_function3: *const c_void,  // 3
-    _untranslated_function4: *const c_void,  // 4
+    free: extern "C" fn(*mut u8),            // 4
     _untranslated_function5: *const c_void,  // 5
     _untranslated_function6: *const c_void,  // 6
     _untranslated_function7: *const c_void,  // 7
@@ -71,7 +85,8 @@ struct Stubs {
     _untranslated_function15: *const c_void, // 15
     _untranslated_function16: *const c_void, // 16
     _untranslated_function17: *const c_void, // 17
-    _untranslated_function18: *const c_void, // 18
+    convert_to_type:
+        extern "C" fn(*const Interpreter, *mut RawObject, *const ObjectType) -> TclStatus, // 18
     _untranslated_function19: *const c_void, // 19
     _untranslated_function20: *const c_void, // 20
     _untranslated_function21: *const c_void, // 21
@@ -82,8 +97,8 @@ struct Stubs {
     _untranslated_function26: *const c_void, // 26
     _untranslated_function27: *const c_void, // 27
     _untranslated_function28: *const c_void, // 28
-    _untranslated_function29: *const c_void, // 29
-    _untranslated_function30: *const c_void, // 30
+    duplicate_obj: extern "C" fn(*const RawObject) -> *mut RawObject, // 29
+    free_obj: extern "C" fn(*mut RawObject), // 30
     _untranslated_function31: *const c_void, // 31
     _untranslated_function32: *const c_void, // 32
     _untranslated_function33: *const c_void, // 33
@@ -93,9 +108,9 @@ struct Stubs {
     _untranslated_function37: *const c_void, // 37
     _untranslated_function38: *const c_void, // 38
     _untranslated_function39: *const c_void, // 39
-    _untranslated_function40: *const c_void, // 40
+    get_obj_type: extern "C" fn(*const u8) -> *const ObjectType, // 40
     _untranslated_function41: *const c_void, // 41
-    _untranslated_function42: *const c_void, // 42
+    invalidate_string_rep: extern "C" fn(*mut RawObject), // 42
     _untranslated_function43: *const c_void, // 43
     _untranslated_function44: *const c_void, // 44
     _untranslated_function45: *const c_void, // 45
@@ -108,7 +123,7 @@ struct Stubs {
     _untranslated_function52: *const c_void, // 52
     _untranslated_function53: *const c_void, // 53
     _untranslated_function54: *const c_void, // 54
-    _untranslated_function55: *const c_void, // 55
+    new_obj: extern "C" fn() -> *mut RawObject, // 55
     _untranslated_function56: *const c_void, // 56
     _untranslated_function57: *const c_void, // 57
     _untranslated_function58: *const c_void, // 58
@@ -225,7 +240,7 @@ struct Stubs {
     _untranslated_function163: *const c_void, // 163
     _untranslated_function164: *const c_void, // 164
     _untranslated_function165: *const c_void, // 165
-    get_obj_result: extern "C" fn(*const Interpreter) -> *const Object, // 166
+    get_obj_result: extern "C" fn(*const Interpreter) -> *mut RawObject, // 166
     _untranslated_function167: *const c_void, // 167
     _untranslated_function168: *const c_void, // 168
     _untranslated_function169: *const c_void, // 169
@@ -270,7 +285,7 @@ struct Stubs {
     _untranslated_function208: *const c_void, // 208
     _untranslated_function209: *const c_void, // 209
     _untranslated_function210: *const c_void, // 210
-    _untranslated_function211: *const c_void, // 211
+    register_obj_type: extern "C" fn(*const ObjectType), // 211
     _untranslated_function212: *const c_void, // 212
     _untranslated_function213: *const c_void, // 213
     _untranslated_function214: *const c_void, // 214
@@ -399,7 +414,7 @@ struct Stubs {
     _untranslated_function337: *const c_void, // 337
     _untranslated_function338: *const c_void, // 338
     _untranslated_function339: *const c_void, // 339
-    get_string: extern "C" fn(*const Object) -> *const c_char, // 340
+    get_string: extern "C" fn(*const RawObject) -> *const c_char, // 340
     _untranslated_function341: *const c_void, // 341
     _untranslated_function342: *const c_void, // 342
     _untranslated_function343: *const c_void, // 343
@@ -489,7 +504,7 @@ struct Stubs {
     _untranslated_function427: *const c_void, // 427
     attempt_alloc: extern "C" fn(usize) -> *mut u8, // 428
     _untranslated_function429: *const c_void, // 429
-    _untranslated_function430: *const c_void, // 430
+    attempt_realloc: extern "C" fn(*mut u8, usize) -> *mut u8, // 430
     _untranslated_function431: *const c_void, // 431
     _untranslated_function432: *const c_void, // 432
     _untranslated_function433: *const c_void, // 433
@@ -700,9 +715,9 @@ struct Stubs {
     _untranslated_function638: *const c_void, // 638
     _untranslated_function639: *const c_void, // 639
     _untranslated_function640: *const c_void, // 640
-    _untranslated_function641: *const c_void, // 641
-    _untranslated_function642: *const c_void, // 642
-    _untranslated_function643: *const c_void, // 643
+    incr_ref_count: Option<extern "C" fn(*mut RawObject)>, // 641
+    decr_ref_count: Option<extern "C" fn(*mut RawObject)>, // 642
+    is_shared: Option<extern "C" fn(*const RawObject) -> isize>, // 643
     _untranslated_function644: *const c_void, // 644
     _untranslated_function645: *const c_void, // 645
     _untranslated_function646: *const c_void, // 646
@@ -711,7 +726,7 @@ struct Stubs {
     _untranslated_function649: *const c_void, // 649
 }
 
-/// Error codes for unwrapping a TCL interpreter.
+/// Error codes for unwrapping a Tcl interpreter.
 ///
 /// These exist primarily for debugging and advanced use-cases.  Unless you
 /// are calling [from_raw](Interpreter::from_raw), you should not need to worry about these.
@@ -724,7 +739,7 @@ pub enum Error {
 }
 
 impl<'a> Interpreter {
-    /// Converts a raw pointer to a TCL interpreter into a Rust reference.
+    /// Converts a raw pointer to a Tcl interpreter into a Rust reference.
     ///
     /// Most users should not need to use this function because a reference is
     /// already passed to the appropriate functions.  This is public because
@@ -745,7 +760,44 @@ impl<'a> Interpreter {
         }
     }
 
-    /// Informs the TCL interpreter that the given package and version is available.
+    /// DO NOT USE THIS FUNCTION.
+    ///
+    /// This function is only public so that it can be called by the
+    /// [module_init](rtea_proc::module_init) macro for derivative RTEA
+    /// projects..
+    pub fn init_global_functions(&self) {
+        unsafe {
+            let stubs = self
+                .stubs
+                .as_ref()
+                .expect("stubs missing after initial check");
+
+            ALLOC = Some(stubs.attempt_alloc);
+            REALLOC = Some(stubs.attempt_realloc);
+            FREE = Some(stubs.free);
+
+            NEW_OBJ = Some(stubs.new_obj);
+            FREE_OBJ = Some(stubs.free_obj);
+            DUPLICATE_OBJ = Some(stubs.duplicate_obj);
+            if let Some(f) = stubs.incr_ref_count {
+                INCR_REF_COUNT = f;
+            }
+            if let Some(f) = stubs.decr_ref_count {
+                DECR_REF_COUNT = f;
+            }
+            if let Some(f) = stubs.is_shared {
+                IS_SHARED = f;
+            }
+            INVALIDATE_STRING_REP = Some(stubs.invalidate_string_rep);
+            GET_STRING = Some(stubs.get_string);
+
+            REGISTER_OBJ_TYPE = Some(stubs.register_obj_type);
+            GET_OBJ_TYPE = Some(stubs.get_obj_type);
+            CONVERT_TO_TYPE = Some(stubs.convert_to_type);
+        }
+    }
+
+    /// Informs the Tcl interpreter that the given package and version is available.
     pub fn provide_package(&self, name: &str, version: &str) -> Result<TclStatus, String> {
         let name =
             CString::new(name).map_err(|_| "unexpected Nul in package version".to_string())?;
@@ -782,14 +834,14 @@ impl<'a> Interpreter {
             argc: usize,
             argv: *const *const i8,
         ) -> TclStatus {
-            let interp = Interpreter::from_raw(i).expect("TCL passed bad interpreter");
+            let interp = Interpreter::from_raw(i).expect("Tcl passed bad interpreter");
             let raw_args = unsafe { std::slice::from_raw_parts(argv, argc) };
             let mut args = Vec::with_capacity(raw_args.len());
             for arg in raw_args {
                 args.push(
                     unsafe { std::ffi::CStr::from_ptr(*arg) }
                         .to_str()
-                        .expect("invalid args from TCL"),
+                        .expect("invalid args from Tcl"),
                 );
             }
 
@@ -821,7 +873,7 @@ impl<'a> Interpreter {
     /// This function attempts to delete the command `name` in the
     /// interpreter.  If it exists, `true` is returned, otherwise `false` is
     /// returned.  An error is only returned when the given `name` contains
-    /// Nul characters and is therefore not a valid TCL string.
+    /// Nul characters and is therefore not a valid Tcl string.
     pub fn delete_command(&self, name: &str) -> Result<bool, String> {
         let name = CString::new(name).map_err(|_| "unexpected Nul in command name".to_string())?;
 
@@ -837,31 +889,29 @@ impl<'a> Interpreter {
     }
 
     /// Get the current result object for the interpreter.
-    pub fn get_obj_result(&self) -> &Object {
+    pub fn get_obj_result(&self) -> Object {
         unsafe {
-            (self
+            RawObject::wrap((self
                 .stubs
                 .as_ref()
                 .expect("stubs missing after initial check")
-                .get_obj_result)(self as *const Interpreter)
-            .as_ref()
-            .expect("TCL should guarantee this is not Null")
+                .get_obj_result)(self as *const Interpreter))
         }
     }
 
-    /// Evaluate a TCL script.
+    /// Evaluate a Tcl script.
     ///
-    /// Evaluates the given string as a TCL script.  If the script return
+    /// Evaluates the given string as a Tcl script.  If the script return
     /// `TclStatus::Error`, then the associated error message is passed back
     /// as `Err`.  Otherwise the last commands return value is passed through
     /// as is.
-    pub fn eval(&self, script: &str) -> Result<TclStatus, String> {
+    pub fn eval(&self, script: &str) -> Result<Object, Object> {
         if script.len() > 1 << 31 {
             return Err(
-                "TCL versions prior to 9.0 do not support scripts greater than 2 GiB".to_string(),
+                Object::new(), // "Tcl versions prior to 9.0 do not support scripts greater than 2 GiB".to_string(),
             );
         }
-        let res = unsafe {
+        let status = unsafe {
             (self
                 .stubs
                 .as_ref()
@@ -873,21 +923,22 @@ impl<'a> Interpreter {
                 0,
             )
         };
-        if res == TclStatus::Error {
-            Err(self.get_string(self.get_obj_result()))
+        let result = self.get_obj_result();
+        if status == TclStatus::Error {
+            Err(result)
         } else {
-            Ok(res)
+            Ok(result)
         }
     }
 
     /// Set the interpreter's current result value.
     ///
     /// When inside command logic, this can be used to set the return value
-    /// visible to the invoking TCL script.
+    /// visible to the invoking Tcl script.
     pub fn set_result(&self, text: &str) {
         let tcl_str = self
             .alloc(text.len() + 1)
-            .expect("propagating memory failure in TCL");
+            .expect("propagating memory failure in Tcl");
 
         tcl_str[..text.len()].copy_from_slice(text.as_bytes());
 
@@ -908,23 +959,10 @@ impl<'a> Interpreter {
         }
     }
 
-    /// Gets the string associated with the TCL object.
-    pub fn get_string(&self, obj: &Object) -> String {
-        let raw = unsafe {
-            // Trusting TCL
-            CStr::from_ptr((self
-                .stubs
-                .as_ref()
-                .expect("stubs missing after initial check")
-                .get_string)(obj))
-        };
-        raw.to_str().expect("Invalid UTF-8 from TCL").to_string()
-    }
-
-    /// Allocates TCL-managed memory.
+    /// Allocates Tcl-managed memory.
     ///
-    /// Allocates memory that is directly managed by TCL.  This is required
-    /// for certain interfaces (e.g., bytes representation of TCL objects)
+    /// Allocates memory that is directly managed by Tcl.  This is required
+    /// for certain interfaces (e.g., bytes representation of Tcl objects)
     /// and convenient for others (e.g., creating a Nul terminated string
     /// from a Rust `String`).
     pub fn alloc(&self, size: usize) -> Option<&mut [u8]> {
@@ -932,7 +970,7 @@ impl<'a> Interpreter {
             return None;
         }
         let ptr = unsafe {
-            // Trusting TCL to handle this correctly (check above can be removed for TCL 9.0)
+            // Trusting Tcl to handle this correctly (check above can be removed for Tcl 9.0)
             (self
                 .stubs
                 .as_ref()
@@ -944,7 +982,7 @@ impl<'a> Interpreter {
             None
         } else {
             unsafe {
-                // We've checked that it is not null and therefore trust TCL
+                // We've checked that it is not null and therefore trust Tcl
                 Some(std::slice::from_raw_parts_mut(ptr, size))
             }
         }
@@ -956,19 +994,19 @@ type CmdDataProc<T> =
 
 /// A wrapper for creating stateful commands.
 ///
-/// The `StatefulCommand` type enables the creation of TCL commands that are
+/// The `StatefulCommand` type enables the creation of Tcl commands that are
 /// stateful.  In other words, they can be modified and carry-forward their
 /// state to future invocations (in contrast to the `create_command` method
-/// of [Interpreter] where the function must either be pure (from TCL's
+/// of [Interpreter] where the function must either be pure (from Tcl's
 /// perspective) or use global state).
 ///
 /// # Example
-/// 
+///
 /// ```rust
 /// use std::cell::RefCell;
 ///
 /// use rtea::*;
-/// 
+///
 /// fn create_stateful_command(interp: &Interpreter) {
 ///     fn cmd(
 ///         interp: &Interpreter,
@@ -978,16 +1016,16 @@ type CmdDataProc<T> =
 ///         let mut val = counter.borrow_mut();
 ///         interp.set_result(&val.to_string());
 ///         *val += 1;
-/// 
+///
 ///         Ok(TclStatus::Ok)
 ///     }
-/// 
+///
 ///     let c = StatefulCommand::new(cmd, RefCell::<usize>::new(0));
 ///     c.attach_command(interp, "counter").unwrap();
 ///     
 ///     for i in 0..10 {
 ///         interp.eval("counter").unwrap();
-///         assert_eq!(i.to_string(), interp.get_string(interp.get_obj_result()));
+///         assert_eq!(i.to_string(), interp.get_obj_result().get_string());
 ///     }
 /// }
 /// ```
@@ -1002,7 +1040,7 @@ impl<T> StatefulCommand<T> {
     /// The creates a new `StatefulComand` with ownership of `data`.  The
     /// underlying implementation should be as thread-safe as the original
     /// implementation of `data`, but care needs to be taken to ensure that
-    /// any concurrency from TCL is safe on `data` (there are no concerns for
+    /// any concurrency from Tcl is safe on `data` (there are no concerns for
     /// `proc`).
     pub fn new(proc: CmdDataProc<T>, data: T) -> StatefulCommand<T> {
         StatefulCommand::<T> {
@@ -1011,7 +1049,7 @@ impl<T> StatefulCommand<T> {
         }
     }
 
-    /// Attaches the `StatefulCommand` to a TCL interpreter.
+    /// Attaches the `StatefulCommand` to a Tcl interpreter.
     ///
     /// This exposes the instantiated `StatefulCommand` to the given
     /// interpreter.  This should allow exposing a command to multiple
@@ -1023,21 +1061,21 @@ impl<T> StatefulCommand<T> {
         let state = Box::new(self);
         let name = CString::new(name).map_err(|_| "unexpected Nul in command name".to_string())?;
 
-        // Simple wrapper of the Rust function and data to work with TCL's API.
+        // Simple wrapper of the Rust function and data to work with Tcl's API.
         fn wrapper_proc<T>(
             state: *const StatefulCommand<T>,
             i: *const Interpreter,
             argc: usize,
             argv: *const *const i8,
         ) -> TclStatus {
-            let interp = Interpreter::from_raw(i).expect("TCL passed bad interpreter");
+            let interp = Interpreter::from_raw(i).expect("Tcl passed bad interpreter");
             let raw_args = unsafe { std::slice::from_raw_parts(argv, argc) };
             let mut args = Vec::with_capacity(raw_args.len());
             for arg in raw_args {
                 args.push(
                     unsafe { std::ffi::CStr::from_ptr(*arg) }
                         .to_str()
-                        .expect("invalid args from TCL"),
+                        .expect("invalid args from Tcl"),
                 );
             }
 
@@ -1052,7 +1090,7 @@ impl<T> StatefulCommand<T> {
         // Simple function to restore the `StatefulCommand` to Rust's
         // understanding to allow Rust's RAII code to kick in.
         fn free_state<T>(state: *mut StatefulCommand<T>) {
-            // This relies on TCL to properly track the command state and
+            // This relies on Tcl to properly track the command state and
             // invoke this at the appropriate moment.  Retaking ownership
             // of the underlying pointer ensures the destructor gets called
             unsafe { Box::<StatefulCommand<T>>::from_raw(state) };
