@@ -3,12 +3,12 @@ use rtea::*;
 #[link(name = "tcl")]
 extern "C" {
     // fn Tcl_FindExecutable(name: *const c_char);
-    fn Tcl_CreateInterp() -> *const Interpreter;
-    fn Tcl_DeleteInterp(interp: *const Interpreter);
+    fn Tcl_CreateInterp() -> *mut Interpreter;
+    fn Tcl_DeleteInterp(interp: *mut Interpreter);
 }
 
 pub struct TestInterpreter {
-    interp: *const Interpreter,
+    interp: *mut Interpreter,
 }
 
 impl Drop for TestInterpreter {
@@ -20,11 +20,11 @@ impl Drop for TestInterpreter {
 impl TestInterpreter {
     pub fn new() -> TestInterpreter {
         let raw_interp = unsafe { Tcl_CreateInterp() };
-        assert_ne!(raw_interp, std::ptr::null());
+        assert_ne!(raw_interp, std::ptr::null_mut());
         TestInterpreter { interp: raw_interp }
     }
 
-    pub fn as_ptr(&self) -> *const Interpreter {
+    pub fn as_ptr(&self) -> *mut Interpreter {
         self.interp
     }
 
